/* 
    Copyright (C) 2004  Mika Raento - Renaud Petit

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    email: mraento@cs.helsinki.fi - petit@cs.helsinki.fi 
*/


//CContextServer.h
#ifndef __CONTEXTSERVER__
#define __CONTEXTSERVER__

#include "ver.h"

#include <e32base.h>

#include "ContextCommon.h"
#include "PresenceInfo.h"
#include "SocketsEngine.h"

class CContextServer : public CServer, /*public MSocketObserver,*/ public MContextServerNotifier
{
public:

	static CContextServer* NewL();
	static CContextServer* NewLC();
	~CContextServer();

	static TInt ThreadFunction(TAny* aNone);
	
	void IncrementSessions();
	void DecrementSessions();

	void RunL();
	TInt CheckedRunError(TInt aError);

	CContextServer(TInt aPriority) ;

	void ConstructL() ;

	static void PanicClient(const RMessage& aMessage, TContextServPanic aReason);
	static void PanicServer(TContextServPanic aReason);
	static void ThreadFunctionL();

	//-------------------------------------------------------------
	// communications ...
	//-------------------------------------------------------------
	
	void TerminateContextServer();
	void ConnectToPresenceServer(const RMessage &aMessage);

//	void HandleRequestPresenceInfo(const RMessage &aMessage);
	void HandleRequestPresenceNotification(const RMessage &aMessage);

	void ConnectToPresenceServer(TDesC & username, TDesC & password, TDesC & server, TUint32 accessPoint);
	void SuspendConnection(const RMessage & aMessage);
	void ResumeConnection(const RMessage & aMessage);
	
	CPresenceInfo  * GetPresenceInfo();
	CSocketsEngine * GetJabberClient();
	
/*public: // from MSocketObserver
	void success(CBase* source);
	void error(CBase* source, TInt code, const TDesC& reason);
	void info(CBase* source, const TDesC& msg);
*/
public: // from MContextServerNotifier
	void NotifyConnected();
	void NotifyNewPresenceInfo(const TDesC & from, const TDesC & info, const TTime& stamp);
	void ReportError(TContextServRqstComplete aErrorType, TDesC & aErrorCode, TDesC & aErrorValue);
	void CancelRequest(RMessage Msg);

private:
	TInt iSessionCount;
	CSharableSession *	NewSessionL(const TVersion &aVersion) const;

	CPresenceInfo *		iPresenceInfo;
	//CConnectionOpener *	iConnectionOpener;
	CSocketsEngine *	iJabberClient;

	TUint32		iAccessPoint;
	TBuf<200>	iUsername;
	TBuf<200>	iPassword;
	TBuf<200>	iServer;

	RMessage iMessageFromBook; TInt iBookThreadId;
	RMessage iMessageFromLog; TInt iLogThreadId;

	void SetBookMessage(RMessage Message);
	void SetLogMessage(RMessage Message);
};

#endif