/* 
    Copyright (C) 2004  Mika Raento - Renaud Petit

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    email: mraento@cs.helsinki.fi - petit@cs.helsinki.fi 
*/


//CContextServerSession.cpp

#include "ContextServerSession.h"
#include "ContextCommon.h"
#include <e32svr.h>
#include <flogger.h>
#include "symbian_auto_ptr.h"

CContextServerSession::CContextServerSession(RThread& aClient, CContextServer& aServer) : CSession(aClient), iServer(aServer)
{
    
}

CContextServerSession::~CContextServerSession()
{
    	iServer.DecrementSessions();
}

void CContextServerSession::ServiceL(const RMessage& aMessage)
{
	switch (aMessage.Function())
	{
		case ETerminateContextServer:
			aMessage.Complete(ERequestCompleted);
			iServer.TerminateContextServer();
			break;
			
		case EConnectToPresenceServer:
			ConnectToPresenceServer(aMessage);
			break;

		case ERequestPresenceInfo:
			HandleRequestPresenceInfo(aMessage);
			break;

		case ERequestPresenceNotification:
			HandleRequestPresenceNotification(aMessage);
			break;

		case EUpdateUserPresence:
			UpdateUserPresence(aMessage);
			break;

		case ESuspendConnection:
			SuspendConnection(aMessage);
			break;

		case EResumeConnection:
			ResumeConnection(aMessage);
			break;
		
		case ECancel:
			iServer.CancelRequest(aMessage);
			aMessage.Complete(0);
			break;

		default :
            	PanicClient(EBadRequest);
	}

}

void CContextServerSession::PanicClient(TInt aPanic) const
{
	Panic(KContextServer, aPanic) ; // Note: this panics the client thread, not server
}

CContextServerSession* CContextServerSession::NewL(RThread& aClient, CContextServer& aServer)
{
	CContextServerSession* self = CContextServerSession::NewLC(aClient, aServer) ;
    	CleanupStack::Pop(self) ;
    	return self ;
}

CContextServerSession* CContextServerSession::NewLC(RThread& aClient, CContextServer& aServer)
{
	CContextServerSession* self = new (ELeave) CContextServerSession(aClient, aServer);
    	CleanupStack::PushL(self) ;
    	self->ConstructL() ;
    	return self ;
}

void CContextServerSession::ConstructL()
{
	iServer.IncrementSessions();
}

//------------------------------------------------------------------------

void CContextServerSession::TerminateContextServer(const RMessage& aMessage)
{
	aMessage.Complete(ERequestCompleted);	
	iServer.TerminateContextServer();
}

void CContextServerSession::ConnectToPresenceServer(const RMessage& aMessage)
{
	if (iServer.GetJabberClient()->Connected())
	{
		aMessage.Complete(ERequestCompleted);		
	}
	else
	{
		iServer.ConnectToPresenceServer(aMessage);
	}
}

void CContextServerSession::HandleRequestPresenceInfo(const RMessage& aMessage)
{
	if ( !(iServer.GetJabberClient()->Connected()) )
	{
		aMessage.Complete(EServerUnreachable);
		return;
	}
		
	iServer.GetPresenceInfo()->SetAllAsNew();
	iServer.HandleRequestPresenceNotification(aMessage);	
}

void CContextServerSession::HandleRequestPresenceNotification(const RMessage& aMessage)
{
	iServer.HandleRequestPresenceNotification(aMessage);
}

void CContextServerSession::DisconnectFromPresenceServer(const RMessage& aMessage)
{
	
}

void CContextServerSession::UpdateUserPresence(const RMessage & aMessage)
{
	TInt len=Client().GetDesLength(aMessage.Ptr0());
	auto_ptr<HBufC> tempPresenceInfo(HBufC::NewL(len));
	TPtr16 bufd=tempPresenceInfo->Des();
	
	ReadL(aMessage.Ptr0(), bufd,0);

	iServer.GetJabberClient()->SendPresenceInfoL(bufd);
	aMessage.Complete(ERequestCompleted);
}

void CContextServerSession::SuspendConnection(const RMessage & aMessage)
{
	iServer.SuspendConnection(aMessage);	
}

void CContextServerSession::ResumeConnection(const RMessage & aMessage)
{
	iServer.ResumeConnection(aMessage);
} 