/* 
    Copyright (C) 2004  Mika Raento - Renaud Petit

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    email: mraento@cs.helsinki.fi - petit@cs.helsinki.fi 
*/


#include "transfer.h"
#include <context_log.rsg>
#include "symbian_auto_ptr.h"
#include <plpvariant.h>
#include "cl_settings.h"
#include <bautils.h>
#include "file_output_base.h"

_LIT(cellid_filen, "c:\\system\\apps\\context_log\\cellid_names.txt");
_LIT(transfer_cellid_filen, "c:\\system\\apps\\context_log\\cellid_names_trans.txt");

TDummyPrompt::TDummyPrompt(bool Delete) : iDelete(Delete)
{
}

void ToPacket(const TDesC& filen, TDes& packet)
{
	TInt dirpos=filen.LocateReverse('\\');
	packet=filen.Left(dirpos);
	packet.Append(_L("\\context"));
	packet.Append(filen.Mid(dirpos));
	TInt extpos=packet.LocateReverse('.');
	if (extpos!=KErrNotFound) {
		packet[extpos]='_';
	}
	packet.Append(_L(".xml"));
}

void ToDel(const TDesC& filen, TDes& del)
{
	TInt dirpos=filen.LocateReverse('\\');
	del=filen.Left(dirpos);
	del.Append(_L("\\context"));
	del.Append(filen.Mid(dirpos));

	TInt extpos=del.LocateReverse('.');
	if (extpos!=KErrNotFound) {
		del[extpos]='_';
	}
	del.Append(_L(".del"));
}

void TDummyPrompt::Prompt(const TDesC& /*FileName*/, MUploadCallBack* CallBack)
{
	CallBack->Back(true, iDelete, 0);
}

CTransferBase::CTransferBase(MApp_context& Context, MUploadPrompt& Prompt) : MContextBase(Context), iPrompt(Prompt)
{
	CALLSTACKITEM(_L("CTransferBase::CTransferBase"));

}

CSendUITransfer::CSendUITransfer(MApp_context& Context) : CTransferBase(Context, iDummyPrompt)
{
	CALLSTACKITEM(_L("CSendUITransfer::CSendUITransfer"));

}


void CTransferBase::ConstructL(i_status_notif* callback, const TDesC& dir1, const TDesC& dir2)
{
	CALLSTACKITEM(_L("CTransferBase::ConstructL"));

	send_dirs=new CDesC16ArrayFlat(5);
	send_leave=new CArrayFixFlat<bool>(5);
	dir_prefix1=dir1;
	if (dir1.Length() && dir1.Right(1).Compare(_L("\\")) ) { dir_prefix1.Append(_L("\\")); }

	TFileName contextdir;
	if (dir_prefix1.Length()) {
		contextdir=dir_prefix1;
		contextdir.Append(_L("context\\"));
		TRAPD(err, BaflUtils::EnsurePathExistsL(Fs(), contextdir));
		// ignore errors, since the drive might not exists, of course
		// there might be other errors which will cause a failure
		// in the packet/del files
	}
	dir_prefix2=dir2;
	if (dir2.Length() && dir2.Right(1).Compare(_L("\\")) ) { dir_prefix2.Append(_L("\\")); }
	if (dir_prefix2.Length()) {
		contextdir=dir_prefix2;
		contextdir.Append(_L("context\\"));
		TRAPD(err, BaflUtils::EnsurePathExistsL(Fs(), contextdir));
	}
	cb=callback;
	iCallBack=CTimeOut::NewL(*this);
}

void CSendUITransfer::ConstructL(i_status_notif* callback, TInt cmdid, const TDesC& dir1, const TDesC& dir2)
{
	CALLSTACKITEM(_L("CSendUITransfer::ConstructL"));

	CTransferBase::ConstructL(callback, dir1, dir2);
	sendui=CSendAppUi::NewL(cmdid, NULL);
}

CSendUITransfer* CSendUITransfer::NewL(MApp_context& Context, i_status_notif* callback, TInt cmdid, 
				       const TDesC& dir1, const TDesC& dir2)
{
	CALLSTACKITEM(_L("CSendUITransfer::NewL"));

	auto_ptr<CSendUITransfer> ret(new (ELeave) CSendUITransfer(Context));
	ret->ConstructL(callback, cmdid, dir1, dir2);
	return ret.release();
}

CTransferBase::~CTransferBase()
{
	CALLSTACKITEM(_L("CTransferBase::~CTransferBase"));

	delete dir;
	delete iPacket8;
	delete send_dirs;
	delete send_leave;
	delete file_names; delete list_file_names;
	delete iCallBack;
}

CSendUITransfer::~CSendUITransfer()
{
	CALLSTACKITEM(_L("CSendUITransfer::NewL"));

	delete sendui;
}

void CSendUITransfer::DisplayMenuL(CEikMenuPane& aMenuPane)
{
	CALLSTACKITEM(_L("CSendUITransfer::DisplayMenuL"));

	sendui->DisplaySendCascadeMenuL(aMenuPane);
}

void CSendUITransfer::DisplaySendMenuL(CEikMenuPane& aMenuPane, TInt pos)
{
	CALLSTACKITEM(_L("CSendUITransfer::DisplaySendMenuL"));

	sendui->DisplaySendMenuItemL(aMenuPane, pos, 
		TSendingCapabilities (0, 100000, 0));
}

void CTransferBase::add_filesL(const TDesC& file, bool leave_last)
{
	CALLSTACKITEM(_L("CTransferBase::add_filesL"));

	TFileName filen;
	filen=dir_prefix1;
	filen.Append(file);
	send_dirs->AppendL(filen);
	send_leave->AppendL(leave_last);

	if (dir_prefix2.Length()) {
		filen=dir_prefix2;
		filen.Append(file);
		send_dirs->AppendL(filen);
		send_leave->AppendL(false);
	}
}

void CTransferBase::FileStep()
{
	TEntry fe;
	TBuf<250> msg;

next_step:
	while (dir_i >= dir_count) {
		if (send_dir_i >= send_dirs->Count()) {
			if (again) {
				again=false;
				send_dir_i=0;
			} else {
				GotFiles();
				return;
			}
		}
		dirname=(*send_dirs)[send_dir_i];
		leave=(*send_leave)[send_dir_i];
		send_dir_i++;

		delete dir; dir=0;
		TInt err=Fs().GetDir(dirname, KEntryAttNormal, ESortByName, dir);
		if (err != KErrNone) {
			// msg.Format(_L("error %d getting dir %S"), err, &dirname);
			// cb->status_change(msg);
			goto next_step;
		}
		dir_count=dir->Count();
		if (leave) --dir_count;
		p.Set(dirname, 0, 0);
		dir_i=0;
	}

	filen=p.DriveAndPath();
	filen.Append((*dir)[dir_i].iName);
	dir_i++;
	TInt err=Fs().Entry(filen, fe);
	if (err==KErrNone) {
		if (fe.iSize>0 && fe.IsArchive()) {
			TFileName packet; ToPacket(filen, packet);
			if (BaflUtils::FileExists(Fs(), packet) ) {
				if (file_names) {
					//msg=_L("in progress ");
					//msg.Append(filen);
				} else {
					list_file_names->AppendL(filen);
					msg=_L("adding file ");
					msg.Append(filen);
				}
			} else {
				if (fe.iModified>iAfterTime) {
					msg=_L("prompting file");
					in_call=true;
					iPrompt.Prompt(filen, this);
					in_call=false;
					return;
				}
			}
		} else if (fe.iSize==0) {
			//msg=_L("size 0");
			Fs().Delete(filen);
		} else {
			//msg=_L("skip file ");
			msg.Append(filen);
		}
	} else {
		msg.Format(_L("error getting file attr %d"), err);
	}
	if (msg.Length()>0) {
		cb->status_change(msg);
	}
	goto next_step;
}

void CTransferBase::DoBackL(bool Upload, bool DeleteFromPhone,
	CXmlBuf* Packet)
{
	if (Upload) {
		if (Packet) {
			if (!iPacket8 || iPacket8->Length() < Packet->Buf().Length()) {
				delete iPacket8; iPacket8=0;
				iPacket8=HBufC8::NewL(Packet->Buf().Length());
			} else {
				iPacket8->Des().Zero();
			}
			TPtr8 p=iPacket8->Des();
			CC()->ConvertFromUnicode(p, Packet->Buf());

			TFileName packet; ToPacket(filen, packet);
			RFile f;
			User::LeaveIfError(f.Replace(Fs(), packet, EFileWrite|EFileShareAny));
			CleanupClosePushL(f);
			User::LeaveIfError(f.Write(*iPacket8));
			CleanupStack::PopAndDestroy();
		}
		if (DeleteFromPhone) {
			TFileName del; ToDel(filen, del);
			RFile f;
			User::LeaveIfError(f.Replace(Fs(), del, EFileWrite|EFileShareAny));
			f.Close();
		}
		list_file_names->AppendL(filen);
	} else if (DeleteFromPhone) {
		Fs().Delete(filen);
	} else {
		Fs().SetAtt(filen, 0, KEntryAttArchive);
	}
}

void CTransferBase::Back(bool Upload, bool DeleteFromPhone,
	CXmlBuf* Packet)
{

	TRAPD(err, DoBackL(Upload, DeleteFromPhone, Packet));

	//TODO: report to user
	if (err!=KErrNone) {
		TBuf<50> msg;
		msg.Format(_L("Error %d in Back"), err);
		cb->error(msg);
	}

	if (in_call) {
		iCallBack->Wait(0);
	} else {
		FileStep();
	}
}

void CTransferBase::expired()
{
	FileStep();
}

void CTransferBase::GetFiles(TTime AfterTime)
{
	CALLSTACKITEM(_L("CTransferBase::GetFiles"));

	if (listing_files) again=true;

	if (!file_names) file_index=0;
	listing_files=true;

	if (send_dirs->Count()==0) {
		GotFiles();
		return;
	}

	iAfterTime=AfterTime;

	delete list_file_names; list_file_names=0;
	list_file_names=new CDesC16ArrayFlat(10);

	send_dir_i=dir_i=0;
	dir_count=-1; delete dir; dir=0;

	FileStep();
}

bool CSendUITransfer::transfer_files(TInt cmdid)
{
	CALLSTACKITEM(_L("CSendUITransfer::transfer_files"));

	if (in_progress) return false;

	iCmdId=cmdid;

	in_progress=true;

	GetFiles();

	return true;
}

void CSendUITransfer::GotFiles()
{
	if (listing_files) {
		listing_files=false;
		if (!file_names) {
			file_names=list_file_names;
			list_file_names=0;
		} else {
			return;
		}
	}

	if (!file_names || file_names->Count()==0) {
		delete file_names; file_names=0;
		in_progress=false;
		cb->finished();
		return;
	}

	sendui->CreateAndSendMessageL (iCmdId, 0, file_names);

	CAknQueryDialog* dlg = new(ELeave) CAknQueryDialog(CAknQueryDialog::ENoTone);
	CleanupStack::PushL(dlg);
	
	_LIT(pr, "Delete files?");
	dlg->SetPromptL(pr);

	CleanupStack::Pop(); //dlg

	if (dlg->ExecuteLD(R_CL_CONFIRMATION_QUERY_DIALOG)) {
		if (cb) cb->status_change(_L("deleting files"));
		for (int i=0; i<file_names->Count(); i++) {
			Fs().Delete((*file_names)[i]);
		}
		if (cb) cb->status_change(_L("deleted files"));
	}

	in_progress=false;

	return;
}

CFtpTransfer* CFtpTransfer::NewL(MApp_context& Context, 
	i_status_notif* callback, const TDesC& dir1, const TDesC& dir2,
		TConnType ConnType, MUploadPrompt& Prompt)
{
	CALLSTACKITEM(_L("CFtpTransfer::NewL"));

	auto_ptr<CFtpTransfer> ret(new (ELeave) CFtpTransfer(Context, Prompt));
	ret->ConstructL(callback, dir1, dir2, ConnType);
	return ret.release();
}

void CFtpTransfer::ConstructL(i_status_notif* callback, const TDesC& dir1, const TDesC& dir2,
		TConnType ConnType)
{
	CALLSTACKITEM(_L("CFtpTransfer::ConstructL"));

	CTransferBase::ConstructL(callback, dir1, dir2);
	iConnType=ConnType;
}

CFtpTransfer::CFtpTransfer(MApp_context& Context, MUploadPrompt& Prompt) : CTransferBase(Context, Prompt)
{
	CALLSTACKITEM(_L("CFtpTransfer::CFtpTransfer"));

	current_state=IDLE;
}

void CFtpTransfer::TransferFiles(const TDesC& Meta, TTime AfterTime)
{
	CALLSTACKITEM(_L("CFtpTransfer::TransferFiles"));

	iMeta=Meta;

	GetFiles(AfterTime);
}

void CFtpTransfer::GotFiles()
{
	if (listing_files) {
		listing_files=false;
		if (!file_names) {
			file_names=list_file_names;
			list_file_names=0;
			file_index=0;
		} else {
			return;
		}
	}

	delete http; http=0;

	if (!file_names || file_names->Count()==0 || file_index>=file_names->Count()) {
		delete file_names; file_names=0;
		cb->finished();
		return;
	}

	http=CHttp::NewL(AppContext(), *this);
	current_state=CONNECTING;
	TInt Iap;
	Settings().GetSettingL(SETTING_IP_AP, Iap);
	TBuf<100> url;
	if (! Settings().GetSettingL(SETTING_UPLOAD_URLBASE, url)) {
		url=DEFAULT_UPLOAD_URLBASE;
	}
	url.Append(_L("put5.pl"));
	http->Connect(Iap, url);
}

CFtpTransfer::~CFtpTransfer()
{
	CALLSTACKITEM(_L("CHttp::NewL"));

	delete iFtpPacket8;
	delete http;
}

void CFtpTransfer::GetFileInfoL(HBufC8*& this_packet, TDes& datetime)
{
	TFileName filen=(*file_names)[file_index];

	TFileName del; ToDel(filen, del);
	if (BaflUtils::FileExists(Fs(), del)) {
		iDeleteThisFile=true;
	} else {
		iDeleteThisFile=false;
	}

	TEntry fe;
	User::LeaveIfError(Fs().Entry(filen, fe));
	format_datetime(datetime, fe.iModified);

	TFileName packet; ToPacket(filen, packet);
	if (BaflUtils::FileExists(Fs(), packet)) {
		RFile f;
		User::LeaveIfError(Fs().Entry(packet, fe));
		if (fe.iSize!=0) {
			if (!iFtpPacket8 || iFtpPacket8->Des().MaxLength() < fe.iSize) {
				delete iFtpPacket8; iFtpPacket8=0;
				iFtpPacket8=HBufC8::NewL(fe.iSize);
			} else {
				iFtpPacket8->Des().Zero();
			}
			User::LeaveIfError(f.Open(Fs(), packet, EFileRead|EFileShareAny));
			CleanupClosePushL(f);
			TPtr8 ptr=iFtpPacket8->Des();
			User::LeaveIfError(f.Read(ptr, fe.iSize));
			CleanupStack::PopAndDestroy();
		} else { 
			if (!iFtpPacket8) iFtpPacket8=HBufC8::NewL(2048);
			else iFtpPacket8->Des().Zero();
		}
		this_packet=iFtpPacket8;
	} else {
		this_packet=0;
	}
}

void CFtpTransfer::SendFile()
{
	current_state=SENDING_FILE;
	HBufC8* this_packet=0;

	TBuf<30> dt;
	TRAPD(err, GetFileInfoL(this_packet, dt));
	if (iMeta.Length()==0) dt.Zero();

	if (err==KErrNone) {
		TBuf<40> msg;
		msg.Format(_L("Sending %d"), file_index);
		cb->status_change(msg);

		http->Store((*file_names)[file_index], dt, this_packet);
	} else {
		iDeleteThisFile=false;
		success(0);
	}
}

void CFtpTransfer::success(CBase* /*source*/)
{
	CALLSTACKITEM(_L("CFtpTransfer::success"));

	TBuf<40> msg;
again:
	switch(current_state) {
	case IDLE:
		delete file_names; file_names=0;
		cb->error(_L("inconsistent state"));
		break;
	case CWDING:
		msg.Format(_L("Sending %d"), file_index);
		cb->status_change(msg);
		SendFile();
		break;
	case SENDING_FILE:
		if (iDeleteThisFile) {
			TFileName d;
			Fs().Delete((*file_names)[file_index]);
			ToDel((*file_names)[file_index], d); Fs().Delete(d);
			ToPacket((*file_names)[file_index], d); Fs().Delete(d);
		} else {
			TFileName d;
			Fs().SetAtt((*file_names)[file_index], 0, KEntryAttArchive);
			ToPacket((*file_names)[file_index], d); Fs().Delete(d);
		}
		++file_index;
		if (file_index<file_names->Count()) {
			msg.Format(_L("Sending %d"), file_index);
			cb->status_change(msg);
			SendFile();
		} else {
			if (list_file_names && list_file_names->Count()>0 && ! listing_files) {
				file_index=0;
				delete file_names; file_names=0;
				file_names=list_file_names; list_file_names=0;
				msg.Format(_L("Sending next %d"), file_index);
				cb->status_change(msg);
				SendFile();
			} else {
				cb->status_change(_L("closing"));
				current_state=CLOSING;
				if (http) http->Close();
			}
		}
		break;
	case CLOSING:
		delete http; http=0;
		current_state=IDLE;
		delete file_names; file_names=0;
		if (list_file_names && ! listing_files) {
			GotFiles();
		} else {
			cb->finished();
		}
		break;
	case CONNECTING:
		if (GetImsi(iImsi)) {
			http->Cwd(iImsi);
			current_state=CWDING;
			goto again;
		} else {
			current_state=CLOSING;
			if (http) http->Close();
			delete file_names; file_names=0;
			cb->error(_L("Cannot get IMSI"));
		}
		break;
	default:
		break;
	}

}

bool CFtpTransfer::GetImsi(TDes& aImsi)
{
	CALLSTACKITEM(_L("CFtpTransfer::GetImsi"));

#ifndef __WINS__
	TPlpVariantMachineId machineId;
	PlpVariant::GetMachineIdL(machineId);
	aImsi=machineId;
#else
	// Return a fake IMEI when working on emulator
	_LIT(KEmulatorImsi, "244050000000000");
	aImsi.Copy(KEmulatorImsi);
#endif
	return true;
}

void CFtpTransfer::error(CBase* source, TInt /*code*/, const TDesC& reason)
{
	CALLSTACKITEM(_L("CFtpTransfer::error"));

	if (current_state==MKDING) {
		// probably exists already, let's try CWDing
		success(source);
	} else if (current_state==SENDING_FILE) {
		// let's try the next file
		file_index++;
		GotFiles();
	} else {
		delete file_names; file_names=0;
		cb->error(reason);
		delete http; http=0;
		current_state=IDLE;
	}
}

void CFtpTransfer::info(CBase* /*source*/, const TDesC& msg)
{
	CALLSTACKITEM(_L("CFtpTransfer::info"));

	cb->status_change(msg);
}

CPeriodicTransfer* CPeriodicTransfer::NewL(MApp_context& Context, int hours, MSocketObserver* callback)
{
	CALLSTACKITEM(_L("CPeriodicTransfer::NewL"));

	auto_ptr<CPeriodicTransfer> ret(new (ELeave) 
		CPeriodicTransfer(Context, hours, callback));
	ret->ConstructL();
	return ret.release();
}

CPeriodicTransfer::~CPeriodicTransfer()
{
	CALLSTACKITEM(_L("CPeriodicTransfer::NewL"));

	delete iTimer;
	delete iFtp;
	delete iCommLog;
}

CPeriodicTransfer::CPeriodicTransfer(MApp_context& Context, int hours, MSocketObserver* callback) : 
MContextBase(Context), iHours(hours), iCb(callback)
{
	CALLSTACKITEM(_L("CPeriodicTransfer::CPeriodicTransfer"));

}


void CPeriodicTransfer::ConstructL()
{
	CALLSTACKITEM(_L("CPeriodicTransfer::ConstructL"));

	iTimer=CTimeOut::NewL(*this);
	iFtp=CFtpTransfer::NewL(AppContext(), this, DataDir(), AppDir(), CFtpTransfer::HTTP, iDummyPrompt);
	iFtp->add_filesL(_L("log*txt"), true);
	iFtp->add_filesL(_L("book*txt"), true);
	iFtp->add_filesL(_L("comm*txt"), false);
	iFtp->add_filesL(_L("rec*amr"), false);
	iFtp->add_filesL(_L("cellid_names.txt"), false);
	iCommLog=Clog_comm::NewL(AppContext(), this);
	current_state=IDLE;
	iTimer->Wait(60*60);
}

void CPeriodicTransfer::expired()
{
	CALLSTACKITEM(_L("CPeriodicTransfer::expired"));

	TTime now;
	now.HomeTime();


	if (++iCount>=iHours || (iHours==24 && now.DateTime().Hour()==3) ) {
			Transfer();
	} else {
		iTimer->Wait(60*60);
        //iTimer->Wait(1);
	}
}

void CPeriodicTransfer::Transfer(bool MakeCommLog)
{
	CALLSTACKITEM(_L("CPeriodicTransfer::Transfer"));

	iTimer->Reset();
	iCb->info(this, _L("Getting comm log"));
	current_state=GETTING_COMM_LOG;
	if (MakeCommLog && ! NoSpaceLeft()) {
		iCommLogFrom=0;
		TTime existing(0);
		TRAPD(err, Settings().GetSettingL(SETTING_LAST_COMMLOG_UPLOAD, iCommLogFrom));
		iCommLog->LogsExist(existing);
		if (existing>iCommLogFrom) iCommLogFrom=existing;
		if (! iCommLog->write_comm_log(iCommLogFrom)) {
			finished();
		}
	} else {
		finished();
	}
}

void CPeriodicTransfer::finished()
{
	CALLSTACKITEM(_L("CPeriodicTransfer::finished"));

	if (current_state==GETTING_COMM_LOG) {
		iCb->info(this, _L("FTPing"));
		current_state=FTPING;
		iFtp->TransferFiles();
	} else {
		iCount=0;
		current_state=IDLE;
		TRAPD(err, Settings().WriteSettingL(SETTING_LAST_COMMLOG_UPLOAD, iCommLogFrom));
		iTimer->Wait(60*60);
		iCb->info(this, _L("Transferred"));
	}
}

void CPeriodicTransfer::error(const TDesC& reason)
{
	CALLSTACKITEM(_L("CPeriodicTransfer::error"));

	current_state=IDLE;
	iTimer->Wait(60*60);
	iCb->error(this, 1, reason);
}

void CPeriodicTransfer::status_change(const TDesC& msg)
{
	CALLSTACKITEM(_L("CPeriodicTransfer::status_change"));

	iCb->info(this, msg);
}
